@@ -1,26 +1,30 @@
 import argparse
 import re
 import sys
-import traceback
-from typing import Any, Iterator, List, Optional, Union
+from typing import Any, Iterator, List, Union
 
 
-# TODO: for this function, I would like to have a decorator that splits the given code_text into code blocks for each function and then creates a sig for each of them
 # @decorators.map_firstp_arg
 def python_functions_signatures(
-    code_text: str, *, ignore_private_functions: bool = False, keep_function_name: bool = False
+    code_text: str,
+    *,
+    ignore_private_functions: bool = False,
+    ignore_nested_functions: bool = False,
+    keep_function_name: bool = False,
 ) -> List[str]:
-    """Return the function signatures (the parameters and their types/default values) for all of the functions in the given code_text."""
+    """Return the function signatures for all of the functions in the given code_text."""
     from d8s_strings import string_remove_from_start
 
     from .ast_data import python_function_names
 
     signatures = []
 
-    function_names = python_function_names(code_text, ignore_private_functions=ignore_private_functions)
+    function_names = python_function_names(
+        code_text, ignore_private_functions=ignore_private_functions, ignore_nested_functions=ignore_nested_functions
+    )
 
     for name in function_names:
-        regex_for_signature = f'(def {name}\((?:.|\s)*?\).*?):'
+        regex_for_signature = fr'(def {name}\((?:.|\s)*?\).*?):'
         sig = re.findall(regex_for_signature, code_text)
         if any(sig):
             new_sig = string_remove_from_start(sig[0], 'def ')
@@ -43,7 +47,10 @@ def python_todos(code_text: str, todo_regex: str = 'TODO:.*') -> List[str]:
 
 # @decorators.map_first_arg
 def python_make_pythonic(name: str) -> str:
-    """Make the name pythonic (e.g. 'fooBar' => 'foo_bar', 'foo-bar' => 'foo_bar', 'foo bar' => 'foo_bar', 'Foo Bar' => 'foo_bar')."""
+    """Make the name pythonic.
+
+    (e.g. 'fooBar' => 'foo_bar', 'foo-bar' => 'foo_bar', 'foo bar' => 'foo_bar', 'Foo Bar' => 'foo_bar').
+    """
     from d8s_strings import lowercase, snake_case, string_split_on_uppercase
 
     split_string = '_'.join(
@@ -87,7 +94,9 @@ def python_clean(code_text: str) -> str:
     return code_text
 
 
-def python_function_blocks(code_text: str, *, ignore_private_functions: bool = False) -> List[str]:
+def python_function_blocks(
+    code_text: str, *, ignore_private_functions: bool = False, ignore_nested_functions: bool = False
+) -> List[str]:
     """Find the code (as a string) for every function in the given code_text."""
     from d8s_lists import has_index
     from d8s_strings import string_chars_at_start_len
@@ -96,7 +105,7 @@ def python_function_blocks(code_text: str, *, ignore_private_functions: bool = F
 
     function_block_strings = []
     code_text_as_lines = code_text.splitlines()
-    ast_function_defs = python_ast_function_defs(code_text)
+    ast_function_defs = python_ast_function_defs(code_text, recursive_search=not ignore_nested_functions)
     function_block_line_numbers = [(f.name, python_ast_object_line_numbers(f)) for f in ast_function_defs]
 
     for function_name, (start, end) in function_block_line_numbers:
@@ -107,7 +116,10 @@ def python_function_blocks(code_text: str, *, ignore_private_functions: bool = F
             if function_name.startswith('_'):
                 continue
 
-        # the code below checks to see if the line after what was determined to be the last line of the function should also be included in the function block (which is the case when the closing parenthesis of a function call in another function is on a newline (see the python_data_tests.py::test_python_function_blocks_edge_cases_1 for an example))
+        # the code below checks to see if the line after what was determined to be the last line of the function...
+        # should also be included in the function block (which is the case when the closing parenthesis of a...
+        # function call in another function is on a newline (see the...
+        # python_data_tests.py::test_python_function_blocks_edge_cases_1 for an example))
         if has_index(code_text_as_lines, end):
             # find the indentation level of the function definition (the first line of the function)
             function_indentation = string_chars_at_start_len(function_block_lines[0], ' ')
@@ -163,7 +175,6 @@ def python_files_using_function(function_name: str, search_path: str) -> List[st
 
     files_using_function = []
 
-    # TODO: this function will return the function definition as well as any uses - I would like to split these two things up
     function_pattern = f'{function_name}('
     python_files = directory_read_files_with_path_matching(search_path, '*.py')
     for file_path, file_contents in python_files:
@@ -173,25 +184,25 @@ def python_files_using_function(function_name: str, search_path: str) -> List[st
     return files_using_function
 
 
-def python_keywords(code_text: Optional[str] = None) -> List[str]:
+def python_keywords() -> List[str]:
     """Get a list of the python keywords."""
     import keyword
 
-    python_keywords = keyword.kwlist
+    return keyword.kwlist
 
-    if code_text is None:
-        return python_keywords
-    else:
-        from d8s_strings import string_words
+    # if code_text is None:
+    #     return python_keywords
+    # else:
+    #     from d8s_strings import string_words
 
-        words_in_code_text = string_words(code_text)
-        keywords_used = []
+    #     words_in_code_text = string_words(code_text)
+    #     keywords_used = []
 
-        for palabra in words_in_code_text:
-            if palabra in python_keywords:
-                keywords_used.append(palabra)
+    #     for palabra in words_in_code_text:
+    #         if palabra in python_keywords:
+    #             keywords_used.append(palabra)
 
-        return keywords_used
+    # return keywords_used
 
 
 # @decorators.map_first_arg
@@ -207,17 +218,17 @@ def python_object_properties_enumerate(
         string_to_eval_as_property = 'python_object.{}'.format(i)
         try:
             if run_methods:
-                eval_result = eval(string_to_eval_as_property)
+                eval_result = eval(string_to_eval_as_property)  # pylint: disable=W0123
                 if callable(eval_result):
                     string_to_eval_as_function = 'python_object.{}()'.format(i)
                     try:
-                        print(f'{i}: {eval(string_to_eval_as_function)}')
+                        print(f'{i}: {eval(string_to_eval_as_function)}')  # pylint: disable=W0123
                     except TypeError:
-                        print(f'{i}: {eval(string_to_eval_as_property)}')
+                        print(f'{i}: {eval(string_to_eval_as_property)}')  # pylint: disable=W0123
                 else:
                     print(f'{i}: {eval_result}')
             else:
-                print(f'{i}: {eval(string_to_eval_as_property)}')
+                print(f'{i}: {eval(string_to_eval_as_property)}')  # pylint: disable=W0123
         except AttributeError:
             print(f'! Unable to get the {i} attribute for the item.')
 
@@ -257,7 +268,10 @@ def python_file_names(path: str, *, exclude_tests: bool = False) -> List[str]:
 
 # @decorators.map_first_arg
 def python_fstrings(code_text: str, *, include_braces: bool = False) -> Iterator[str]:
-    """Find all of the python formatted string literals in the given text. See https://realpython.com/python-f-strings/ for more details about f-strings."""
+    """Find all of the python formatted string literals in the given text.
+
+    See https://realpython.com/python-f-strings/ for more details about f-strings.
+    """
     from d8s_grammars import python_formatted_string_literal
     from d8s_lists import flatten
 
