@@ -8,7 +8,7 @@ isort d8s_python/ tests/
 
 black d8s_python/ tests/
 
-mypy d8s_python/ tests/
+# mypy d8s_python/ tests/
 
 pylint --fail-under 9 d8s_python/*.py
 
