@@ -1,7 +1,4 @@
 import ast
-import os
-
-from d8s_file_system import file_read
 
 from d8s_python import (
     python_ast_exception_handler_exceptions_raised,
@@ -30,6 +27,7 @@ async def foo():
     """."""
     print('Here!')
 
+
 def python_function_names(code_text: str, *, ignore_private_functions: bool = False) -> List[str]:
     """."""
     function_objects = python_ast_function_defs(code_text)
@@ -94,6 +92,26 @@ TEST_FUNCTION_WITH_DEFAULT = '''def test(a: str = ''):
     """Docstring."""
     return a'''
 
+TEST_CODE_WITH_NESTED_FUNCTION = '''
+def f(n):
+    """a."""
+    def sq(i):
+        """b."""
+        return i * i
+    return sq(n)
+'''
+
+TEST_CODE_WITH_ASYNC_FUNCTION = '''
+def foo(n):
+    """Foo."""
+    return n
+
+
+async def bar(n):
+    """Some async func."""
+    return n
+'''
+
 TEST_EXCEPTION_DATA = [
     {  # test a simple raise
         'code': '''raise ValueError('I cannot divide by zero')''',
@@ -255,6 +273,16 @@ def test_python_ast_objects_of_type_1():
     assert isinstance(result[0], ast.FunctionDef)
 
 
+def test_python_ast_objects_of_type__only_first_level():
+    # test without limiting search to the first level
+    assert len(tuple(python_ast_objects_of_type(TEST_CODE_WITH_NESTED_FUNCTION, ast.FunctionDef))) == 2
+
+    # test limiting search to the first level
+    result = tuple(python_ast_objects_of_type(TEST_CODE_WITH_NESTED_FUNCTION, ast.FunctionDef, recursive_search=False))
+    assert len(result) == 1
+    assert isinstance(result[0], ast.FunctionDef)
+
+
 def test_python_exceptions_handled_docs_1():
     for test in TEST_EXCEPTION_DATA:
         try:
@@ -292,10 +320,9 @@ def test_python_exceptions_raised_docs_1():
 
 
 def test_python_functions_as_import_string_1():
-    print(python_functions_as_import_string(TEST_CODE, 'ast_data'))
     assert (
         python_functions_as_import_string(TEST_CODE, 'ast_data')
-        == 'from ast_data import (\n    python_function_names,\n    python_function_docstrings,\n    python_variable_names,\n    python_constants,\n)'
+        == 'from ast_data import (\n    python_function_names,\n    python_function_docstrings,\n    python_variable_names,\n    python_constants,\n    foo,\n)'
     )
 
 
@@ -366,11 +393,21 @@ def test_python_function_names_1():
         'python_function_docstrings',
         'python_variable_names',
         'python_constants',
+        'foo',
     ]
     assert python_function_names(TEST_CODE_WITH_PRIVATE_FUNCTION) == ['_test']
     assert python_function_names(TEST_CODE_WITH_PRIVATE_FUNCTION, ignore_private_functions=True) == []
 
 
+def test_python_function_names__nested_functions():
+    assert python_function_names(TEST_CODE_WITH_NESTED_FUNCTION) == ['f', 'sq']
+    assert python_function_names(TEST_CODE_WITH_NESTED_FUNCTION, ignore_nested_functions=True) == ['f']
+
+
+def test_python_function_names__async_functions():
+    assert python_function_names(TEST_CODE_WITH_ASYNC_FUNCTION) == ['foo', 'bar']
+
+
 def test_python_function_docstrings_1():
     code_text = '''def _test(a: str):
     """Docstring."""
@@ -379,6 +416,15 @@ def test_python_function_docstrings_1():
     assert python_function_docstrings(code_text, ignore_private_functions=True) == []
 
 
+def test_python_function_docstrings__nested_functions():
+    assert python_function_docstrings(TEST_CODE_WITH_NESTED_FUNCTION) == ['a.', 'b.']
+    assert python_function_docstrings(TEST_CODE_WITH_NESTED_FUNCTION, ignore_nested_functions=True) == ['a.']
+
+
+def test_python_function_docstrings__async_functions():
+    assert python_function_docstrings(TEST_CODE_WITH_ASYNC_FUNCTION) == ['Foo.', 'Some async func.']
+
+
 def test_python_variable_names_1():
     assert python_variable_names('x = 7') == ['x']
     assert python_variable_names('x = y + 7') == ['x']
